program linco;
{
    LineInCode - records an audio stream and writes it to the StdOut.
    Copyright (C) 2003 Roman Mamedov (roman@rm.pp.ru, http://rm.pp.ru/?en)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, Roman Mamedov gives permission to link
    the code of this program with the BASS library v1.5(BASS.DLL) and the
    Delphi API to that library(BASS.PAS) (or with modified versions of
    BASS.DLL and BASS.PAS that use the same license as BASS library v1.5), and
    distribute linked combinations including them.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than BASS.DLL and BASS.PAS.  If you modify this file, you may extend
    this exception to your version of the file, but you are not obligated to
    do so.  If you do not wish to do so, delete this exception statement from
    your version.
}

uses
  Windows,
  SysUtils,
  bass,
  unRM_ConsoleTools,
  unRM_ParseCommandLine,
  unCommonTypes;

{$APPTYPE CONSOLE}

var
  F:File;
  CP: TRMCommandLineParser;
  Terminate, useStopTime: Boolean;
  setBits, setChannels, bassRate, bassFlags: Integer;
  setRecordTime, SysTimeFormat: String;
  StopTime:TDateTime; 
var
  beof:Byte = 26;

procedure FatalError(const Msg: AnsiString);
begin
  MessageBox(0, PChar(Msg), PChar(PROGNAME + ': Fatal Error'), MB_OK or MB_ICONSTOP);
  Halt;
end;

function BassCallback(buffer: Pointer; length: DWORD; user: DWORD): BOOL;
  stdcall;
begin
  Result := True;
  try
    BlockWrite(F, buffer^, length);
  except
    FatalError('Error writing to StdOut.'#13#10'Please terminate "' + EXENAME
      + '.exe" using the Task Manager(Win2000/XP) or Ctrl-Alt-Del(Win9x/ME).');
  end;
end;

begin
  if CommandLineIsHelpRequest then
  begin
    PrintProgramInfo(PROGNAME, PROGVER, RELDATE,
      '(C) 2003 Roman Mamedov.', AUTHMAIL,
      WEBSITE, WEBSITERUS, PROGDESC,'LICENSE.txt');
    WriteStringArray(USAGEINFO);
    Exit;
  end;
  Terminate := False;
  CP := TRMCommandLineParser.Create;
  CP.AddParameter('-B', '--bits', setBits, 16);
  CP.AddParameter('-C', '--channels', setChannels, 2);
  CP.AddParameter('-R', '--rate', bassRate, 44100);
  CP.AddParameter('-D', '--duration', setRecordTime, '');
  CP.Parse;
  CP.Free;
  bassFlags := 0;
  if setBits = 8 then Inc(bassFlags, BASS_SAMPLE_8BITS);
  if setChannels = 1 then Inc(bassFlags, BASS_SAMPLE_MONO);
  AssignFile(F, '');
  try
    Rewrite(F, 1);
  except
    FatalError('Unable to open StdOut.');
  end;
  if not BASS_RecordInit(-1) then
    FatalError('Unable to initialize audio device.');
  useStopTime := not(setRecordTime = '');
  StopTime := 0;
  SysTimeFormat := LongTimeFormat;
  LongTimeFormat := 'h:m:s'; //temporarily changing time format
  if useStopTime then
    try
      StopTime := Now + StrToTime(setRecordTime);
    except
      useStopTime := False;
    end;
  LongTimeFormat := SysTimeFormat;
  if not BASS_RecordStart(bassRate, bassFlags, @BassCallback, 0) then
    FatalError('Unable to start recording.');
  while (not Terminate) do begin
    Sleep(100);
    if useStopTime then begin
      if Now > StopTime then Break;
    end;
  end;
  BASS_ChannelStop(RECORDCHAN);
  BlockWrite(F, beof, 1);
  CloseFile(F);
end.

