unit unRM_ConsoleTools;
{
    unRM_ConsoleTools - an utility unit for working with console and command-line.
    Copyright (C) 2003 Roman Mamedov (roman@rm.pp.ru, http://rm.pp.ru/?en)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}

interface

uses SysUtils;

type BOOL = LongBool;

function CharToOem(lpszSrc: PChar; lpszDst: PChar): BOOL; stdcall;
function CommandLineIsHelpRequest: Boolean;
function OemToChar(lpszSrc: PChar; lpszDst: PChar): BOOL; stdcall;
procedure PrintProgramInfo(const ProgramName, Version, ReleaseDate, Copyright, EMail,
  WebSite, WebSiteRU, Info, GPLfilename: string);
function ReadInt(const vName: string): Integer;
procedure ReadIntArray(const aName: string; var A: array of Integer);
function ReadLnA: string;
function ReadReal(const vName: string): Real;
function StrToInt(const S: string): Integer;
procedure WriteA(const Line: string);
procedure WriteLnA(const Line: string);
procedure WriteLns(Count:Integer);
procedure WriteStringArray(const A: array of string);

var
  RMCONSOLETOOLS_LANG: Integer = 0;

implementation

function OemToChar; external 'user32.dll' name 'OemToCharA';
function CharToOem; external 'user32.dll' name 'CharToOemA';

procedure PrintProgramInfo(const ProgramName, Version, ReleaseDate, Copyright,
  EMail,
  WebSite, WebSiteRU, Info, GPLfilename: string);
begin
  WriteLn(ProgramName + ' v' + Version + ' (' + ReleaseDate + ')', #13#10);
  if (Info <> '') then
    WriteLn(Info, #13#10);
  WriteLn(Copyright);
  WriteLn('E-Mail: ' + EMail);
  if GPLfilename <> '' then
  begin
    WriteLn(#13#10'This program is free software; you can redistribute it and/or modify');
    WriteLn('it under the terms of the GNU General Public License as published by');
    WriteLn('the Free Software Foundation; either version 2 of the License, or');
    WriteLn('(at your option) any later version. See "' + GPLfilename +
      '" for details.');
  end;
  if (WebSite <> '') or (WebSiteRU <> '') then
  begin
    WriteLn(#13#10, 'HOMEPAGE: ');
    if WebSite <> '' then WriteLn('  English -> ' + WebSite);
    if WebSiteRU <> '' then WriteLn('  Russian -> ' + WebSiteRU);
  end;
  WriteLn;
end;

procedure WriteLns(Count:Integer);
var // Prints Count of newlines
  i: Integer;
begin
  if Count > 0 then
    for i := 1 to Count do
      WriteLn;
end;

function ReadLnA: string;
var // Input data from keyboard(incl. Russian characters)
  Temp: PChar;
begin
  ReadLn(Result);
  if not(Result = '') then begin
    Temp := PChar(Copy(Result, 0, Length(Result)));
    OemToChar(Temp, Temp);
    Result := Temp;
  end;
end;

procedure ReadIntArray(const aName: string; var A: array of Integer);
var // Input an array of Integer numbers
  I: Integer;
begin
  for I := 0 to High(A) do
    A[I] := ReadInt(aName + '[' + IntToStr(I + 1) + ']');
end;

function ReadInt(const vName: string): Integer;
//       Input an integer number
begin // vName - a message to the user
  case RMCONSOLETOOLS_LANG of
    0: WriteA(' ' + vName + ',  Integer: ');
    1: WriteA('Please enter ' + vName + '(type - Integer): ');
  end;
  ReadLn(Result);
end;

function ReadReal(const vName: string): Real;
// Input a float number
// vName - a message to the user
begin
  case RMCONSOLETOOLS_LANG of
    0: WriteA(' ' + vName + ',  Real: ');
    1: WriteA('Please enter ' + vName + '(type - Real): ');
  end;
  ReadLn(Result);
end;

procedure WriteLnA(const Line: string);
var // Print text to screen(incl. Russian characters), followed by newlines
  Temp: PChar;
begin
  Temp := PChar(Copy(Line, 0, Length(Line)));
  CharToOem(Temp, Temp);
  WriteLn(Temp);
end;

procedure WriteA(const Line: string);
var // Print text to screen(incl. Russian characters)
  Temp: PChar;
begin
  Temp := PChar(Copy(Line, 0, Length(Line)));
  CharToOem(Temp, Temp);
  Write(Temp);
end;

function StrToInt(const S: string): Integer;
// this is here just to avoid "uses SysUtils"(in some cases)
// and reduce size of the executable
var
  E: Integer;
begin
  Val(S, Result, E);
end;

function CommandLineIsHelpRequest: Boolean;
var
  PStr1: string;
begin
  Result := ParamCount = 0;
  if not Result then
  begin
    PStr1 := ParamStr(1);
    Result := (ParamCount = 1) and
      ((PStr1 = '-h') or (PStr1 = '--help') or (PStr1 = '-?') or (PStr1 = '--?')
      or (PStr1 = '/?') or (PStr1 = '/h') or (PStr1 = '/help'));
  end;
end;

procedure WriteStringArray(const A: array of string);
var
  C: Integer;
begin
  for C := Low(A) to High(A) do
    WriteLn(A[C]);
end;

end.

