unit unRM_ParseCommandLine;
{
    unRM_ParseCommandLine - an utility unit for parsing command-line parameters.
    Copyright (C) 2003 Roman Mamedov (roman@rm.pp.ru, http://rm.pp.ru/?en)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}
interface

uses SysUtils, Classes;

type
  TRMCommandLineParameterType = (ptBoolean, ptInteger, ptString);
  TRMCommandLineParameter = record
    ParamType: TRMCommandLineParameterType;
    ParamVar: Pointer;
    ParamName, ParamAlias: string;
  end;
  PRMCommandLineParameter = ^TRMCommandLineParameter;
  TRMCommandLineParser = class(TObject)
    constructor Create;
    destructor Free;
    procedure AddParameter(const Parameter, Alias: string; var Variable: Boolean; Default:
      Boolean); overload;
    procedure AddParameter(const Parameter, Alias: string; var Variable: Integer; Default:
      Integer); overload;
    procedure AddParameter(const Parameter, Alias: string; var Variable: string; Default:
      string); overload;
    function Parse: TStringList;
  private
    FParamList: TList;
    procedure IAddParameter(const Parameter, Alias: string; Variable: Pointer; ParameterType:
      TRMCommandLineParameterType);
  end;

implementation

destructor TRMCommandLineParser.Free;
{$IFNDEF FPC}
var C:Integer;
{$ENDIF}
begin
{$IFNDEF FPC} // following doesn't compile in FPC
  if FParamList.Count > 0 then
    for C:=0 to FParamList.Count-1 do Dispose(FParamList.Items[C]);
{$ENDIF}
  FParamList.Free;
end;

constructor TRMCommandLineParser.Create;
begin
  FParamList := TList.Create;
end;

function TRMCommandLineParser.Parse: TStringList;
var
  C, D: Integer;
  CurrentParam: string;
  Recognized: Boolean;
  CP: PRMCommandLineParameter;
begin
  C := 1;
  Result := TStringList.Create;
  while C <= ParamCount do
  begin
    CurrentParam := ParamStr(C);
    Recognized := False;
    for D := 0 to FParamList.Count - 1 do
    begin
      CP := FParamList.Items[D];
      if (CP^.ParamName = CurrentParam) or (
      (CP^.ParamAlias <> '') and (CP^.ParamAlias = CurrentParam)
      ) then
      begin
        Recognized := True;
        case CP^.ParamType of
          ptBoolean: Boolean(CP^.ParamVar^) := not Boolean(CP^.ParamVar^);
          ptInteger:
            try
              Inc(C);
              Integer(CP^.ParamVar^) := StrToInt(ParamStr(C));
            except
            end;
          ptString:
            begin
              Inc(C);
              string(CP^.ParamVar^) := ParamStr(C);
            end;
        end;
      end;
    end;
    if not Recognized then
      Result.Add(CurrentParam);
    Inc(C);
  end;
end;

procedure TRMCommandLineParser.IAddParameter(const Parameter, Alias: string; Variable:
  Pointer; ParameterType: TRMCommandLineParameterType);
var CP: PRMCommandLineParameter;  
begin
  New(CP);
  CP^.ParamType := ParameterType;
  CP^.ParamVar := Variable;
  CP^.ParamName := Parameter;
  CP^.ParamAlias := Alias;
  FParamList.Add(CP);
end;

procedure TRMCommandLineParser.AddParameter(const Parameter, Alias: string; var Variable:
  string; Default: string);
begin
  Variable := Default;
  IAddParameter(Parameter, Alias, @Variable, ptString);
end;

procedure TRMCommandLineParser.AddParameter(const Parameter, Alias: string; var Variable:
  Boolean; Default: Boolean);
begin
  Variable := Default;
  IAddParameter(Parameter, Alias, @Variable, ptBoolean);
end;

procedure TRMCommandLineParser.AddParameter(const Parameter, Alias: string; var Variable:
  Integer; Default: Integer);
begin
  Variable := Default;
  IAddParameter(Parameter, Alias, @Variable, ptInteger);
end;

end.

